/*
 * Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 *
 */

/** 
 * @brief BLE Heart Rate and Running speed Rely application main file.
 *
 * @detail This application demonstrates a simple "Relay". 
 * Meaning we pass on the values that we receive. By combining a collector part on 
 * one end and a sensor part on the other, we show that the s130 can function
 * simultaneously as a central and a peripheral device.
 *
 * In the figure below, the sensor ble_app_hrs connects and interacts with the relay
 * in the same manner it would connect to a heart rate collector. In this case, the Relay
 * application acts as a central.
 *
 * On the other side, a collector (such as Master Control panel or ble_app_hrs_c) connects
 * and interacts with the relay the same manner it would connect to a heart rate sensor peripheral.
 *
 * Led layout:
 * LED 1: Central side is scanning       LED 2: Central side is connected to a peripheral
 * LED 3: Peripheral side is advertising LED 4: Peripheral side is connected to a central
 *
 * @note While testing, be careful that the Sensor and Collector are actually connecting to the Relay,
 *       and not directly to each other!
 *
 *    Peripheral                  Relay                    Central
 *    +--------+        +-----------|----------+        +-----------+
 *    | Heart  |        | Heart     |   Heart  |        |           |
 *    | Rate   | -----> | Rate     -|-> Rate   | -----> | Collector |
 *    | Sensor |        | Collector |   Sensor |        |           |
 *    +--------+        +-----------|   and    |        +-----------+
 *                      | Running   |   Running|
 *    +--------+        | Speed    -|-> Speed  |
 *    | Running|------> | Collector |   Sensor |
 *    | Speed  |        +-----------|----------+
 *    | Sensor |
 *    +--------+
 */

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf_sdm.h"
#include "ble.h"
#include "ble_hci.h"
#include "softdevice_handler.h"
#include "app_error.h"
#include "boards.h"
#include "nrf_gpio.h"
#include "bsp_btn_ble.h"
#include "ble_advdata.h"

#define NON_CONNECTABLE_ADV_INTERVAL    MSEC_TO_UNITS(3000, UNIT_0_625_MS)
#define PERIPHERAL_DEVICE_NAME                          "S130_ADV+SCAN"

static ble_gap_adv_params_t m_adv_params;

static ble_gap_scan_params_t m_scan_param;

//Address of other device, from Master Control Panel. 
// D6:49:06:FE:AB:22
static uint8_t search[BLE_GAP_ADDR_LEN] = {0x22, 0xAB, 0xFE, 0x06, 0x49, 0xD6};
uint8_t		start = 1;
uint8_t		* p_start = &start;



static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    const ble_gap_evt_t   * p_gap_evt = &p_ble_evt->evt.gap_evt;
    

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_ADV_REPORT:
        {
            if( p_gap_evt->params.adv_report.peer_addr.addr[0] == search[0] && \
                p_gap_evt->params.adv_report.peer_addr.addr[1] == search[1] && \
                p_gap_evt->params.adv_report.peer_addr.addr[2] == search[2] && \
                p_gap_evt->params.adv_report.peer_addr.addr[3] == search[3] && \
                p_gap_evt->params.adv_report.peer_addr.addr[4] == search[4] && \
                p_gap_evt->params.adv_report.peer_addr.addr[5] == search[5])
            {
                if (* p_start <= 2)
								{
									nrf_gpio_pin_toggle(LED_RGB_RED);
								}
								else 
									sd_ble_gap_scan_stop();
								
								* p_start = (* p_start) + 1;
							 
            }
        }
        default:
            break;
    }
}

static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    on_ble_evt(p_ble_evt);
}

static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_sec_mode_t sec_mode;
    
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    
    err_code = sd_ble_gap_device_name_set(&sec_mode, (const uint8_t *)PERIPHERAL_DEVICE_NAME, strlen(PERIPHERAL_DEVICE_NAME));
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;

    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_50_PPM, NULL);

    // Enable BLE stack.
    ble_enable_params_t ble_enable_params;
    memset(&ble_enable_params, 0, sizeof(ble_enable_params));
#ifdef S130
    ble_enable_params.gatts_enable_params.attr_tab_size   = BLE_GATTS_ATTR_TAB_SIZE_DEFAULT;
#endif
	
    ble_enable_params.gatts_enable_params.service_changed = false;
	
#ifdef S120
    ble_enable_params.gap_enable_params.role              = BLE_GAP_ROLE_CENTRAL;
#endif

    err_code = sd_ble_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
}



/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    uint32_t err_code; 
    ble_advdata_t advdata;
    
    memset(&advdata, 0, sizeof(advdata));
    
    advdata.name_type             = BLE_ADVDATA_FULL_NAME;
    advdata.flags            = BLE_GAP_ADV_FLAG_BR_EDR_NOT_SUPPORTED;

    err_code = ble_advdata_set(&advdata, NULL);
    APP_ERROR_CHECK(err_code);

    // Initialize advertising parameters (used when starting advertising).
    memset(&m_adv_params, 0, sizeof(m_adv_params));

    m_adv_params.type        = BLE_GAP_ADV_TYPE_ADV_NONCONN_IND;
    m_adv_params.interval    = NON_CONNECTABLE_ADV_INTERVAL;
}


/** @brief Function for the Power manager.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();

    APP_ERROR_CHECK(err_code);
}

static void scan_init(void)
{
    m_scan_param.active = 0;
    m_scan_param.selective = 0;
    m_scan_param.p_whitelist = NULL;
    m_scan_param.timeout = 0;
    m_scan_param.interval = 0x00A0;   // 100 ms
    m_scan_param.window = 0x009C;
}

static void scan_start(void)
{  
    uint32_t err_code;
    err_code = sd_ble_gap_scan_start(&m_scan_param);
    APP_ERROR_CHECK(err_code);
}

static void advertising_start(void)
{
    uint32_t err_code;

    err_code = sd_ble_gap_adv_start(&m_adv_params);
    APP_ERROR_CHECK(err_code);
}

int main(void)
{
  
    ble_stack_init();
    gap_params_init();
    advertising_init();
    
    scan_init();
    scan_start();
		
		nrf_gpio_cfg_output(LED_RGB_RED);
	
		while (* p_start <= 2)
		{
			if (* p_start == 4)
			{
		nrf_gpio_pin_clear(LED_RGB_RED);}
				
		}
			
		nrf_gpio_cfg_output(0);
		nrf_gpio_pin_set(0);
		
		nrf_gpio_pin_set(LED_RGB_RED);
				
		advertising_start();
		
		
    for (;; )
    {
        power_manage();
    }
}


